"""
Name: EK_AF_UCS_AttributeTracking.py
Author: Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA
Contact: ryan.j.urbanowicz@darmouth.edu
Created: Sept. 4, 2012
---------------------------------------------------------------------------------------------------------------------------------------------------------
EK_AF_UCS Version 2.0: An expanded learning classifier system algorithm implemented with our new rule compaction strategies for data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRENTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

from EK_AF_UCS_Constants import *
import copy

class AttributeTracking:
    def __init__(self, doAttTrack):
        """ Initialize Attribute Tracking Object"""
        self.activeAttFeedback = False
        if doAttTrack:
            self.probabilityList = []
            if cons.adjustedAT:
                self.attAccuracySumsAdj = [[0]*cons.env.formatData.numAttributes for i in range(cons.env.formatData.numTrainInstances)]
                #self.attCountsWPunish = [[0]*cons.env.formatData.numAttributes for i in range(cons.env.formatData.numTrainInstances)]
            else:
                self.attAccuracySums = [[0]*cons.env.formatData.numAttributes for i in range(cons.env.formatData.numTrainInstances)]
                #self.attCounts = [[0]*cons.env.formatData.numAttributes for i in range(cons.env.formatData.numTrainInstances)]
            if cons.doPopulationReboot:
                self.rebootAT()
    #--------------------------------------------------------------------------------------------------------------------------------------

    def activateAttFeedback(self):
        """ Turns attribute tracking on or off."""
        self.activeAttFeedback = True
        
        
    def deactivateAttFeedback(self):
        """ Turns attribute tracking on or off."""
        self.activeAttFeedback = False
        
        
    def updateAttTrack(self, pop):
        """ Attribute Tracking update."""
        dataRef = cons.env.dataRef
        if cons.adjustedAT:
            for ref in pop.correctSet:
                for each in pop.popSet[ref].specifiedAttList:
                    ruleAccuracy = pop.popSet[ref].getAccuracy()
                    if ruleAccuracy > 0.5:
                        #self.attAccuracySums[dataRef][each] += (pop.popSet[ref].getAccuracy()) #Add rule accuracy
                        self.attAccuracySumsAdj[dataRef][each] += (pop.popSet[ref].fitness) #Add rule accuracy
        else:
            #Add Accuracy to Attributes in rules of the correct set
            for ref in pop.correctSet:
                for each in pop.popSet[ref].specifiedAttList:
                    self.attAccuracySums[dataRef][each] += (pop.popSet[ref].getAccuracy()) #Add rule accuracy
                    #self.attCounts[dataRef][each] += 1

    
    def getTrackProb(self):
        """ Returns the tracking probability list. """
        return self.probabilityList
       
       
    def genTrackProb(self):
        """ Calculate and return the attribute probabilities based on the attribute tracking scores. """
        trackList = []
        #Get data set reference.
        currentInstance = cons.env.dataRef
        if cons.adjustedAT:
            trackList = copy.deepcopy(self.attAccuracySumsAdj[currentInstance])
        else:
            trackList = copy.deepcopy(self.attAccuracySums[currentInstance])
        #----------------------------------------
        minVal = min(trackList)
        for i in range(len(trackList)):
            trackList[i] = trackList[i] - minVal
        maxVal = max(trackList)
        #----------------------------------------
        probList = []
        for i in range(cons.env.formatData.numAttributes):
            if maxVal == 0.0:
                probList.append(0.5)
            else:
                probList.append(trackList[i]/float(maxVal + maxVal*0.01))  #perhaps make this float a constant, or think of better way to do this.
        self.probabilityList = probList
      
      
    def sumGlobalAttTrack(self):
        """ For each attribute, sum the attribute tracking scores over all instances. For Reporting and Debugging"""
        globalAttTrack = [0.0 for i in range(cons.env.formatData.numAttributes)]
        if cons.adjustedAT:
            for i in range(cons.env.formatData.numAttributes):
                for j in range(cons.env.formatData.numTrainInstances):
                    globalAttTrack[i] += self.attAccuracySumsAdj[j][i]     
        else:
            for i in range(cons.env.formatData.numAttributes):
                for j in range(cons.env.formatData.numTrainInstances):
                    globalAttTrack[i] += self.attAccuracySums[j][i]
        return globalAttTrack


    def rebootAT(self):
        """ Rebuilds attribute tracking scores from previously stored run. """
        if cons.env.formatData.areInstanceIDs:
            #----------------------------------------------------
            try: #Obtain existing attribute tracking
                f = open(cons.popRebootPath+"_AttTrack.txt", 'r')
                junkList = f.readline().rstrip('\n').split('\t')
                ATList = []
                for line in f:
                    lineList = line.strip('\n').split('\t')
                    ATList.append(lineList)
                f.close()
                         
            except IOError, (errno, strerror):
                print ("I/O error(%s): %s" % (errno, strerror))
                raise
            #----------------------------------------------------
            #Reorder old att-track values to match new data shuffling.
            dataLink = cons.env.formatData
            for i in range(dataLink.numTrainInstances):
                targetID = dataLink.trainFormatted[i][2] #gets each instance ID
                notFound = True
                j = 0
                while notFound and j < dataLink.numTrainInstances:
                    if str(targetID) == str(ATList[j][0]): #found relevant instance
                        for w in range(dataLink.numAttributes):
                            if cons.adjustedAT:
                                self.attAccuracySumsAdj[i][w] = float(ATList[j][w+1])
                            else:
                                self.attAccuracySums[i][w] = float(ATList[j][w+1])
                    j += 1              
        else:
            print "AttributeTracking: Error - Reboot of Attribute Tracking is only possible with Instance ID's!"
            pass
        