"""
Name: EK_AF_UCS_ClassAccuracy.py
Author: Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA
Contact: ryan.j.urbanowicz@darmouth.edu
Created: Sept. 4, 2012
---------------------------------------------------------------------------------------------------------------------------------------------------------
EK_AF_UCS Version 2.0: An expanded learning classifier system algorithm implemented with our new rule compaction strategies for data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

class ClassAccuracy:
    def __init__(self):
        """ Initialize the accuracy calculation for a single class """
        self.T_myClass = 0      #true positive for binary class problems
        self.T_otherClass = 0   #true negative for binary class problems
        self.F_myClass = 0      #false positive for binary class problems
        self.F_otherClass = 0   #false negative for binary class problems


    def updateAccuracy(self, thisIsMe, accurateClass):
        """ Increment the appropriate cell of the confusion matrix """
        if thisIsMe and accurateClass:
            self.T_myClass += 1
        elif accurateClass:
            self.T_otherClass += 1
        elif thisIsMe:
            self.F_myClass += 1
        else:
            self.F_otherClass += 1
        
        
    def getClassSensitivity(self):
        """ Return class sensitivity, i.e. Measure of the ability to correctly identify members of this class."""
        if self.T_myClass + self.F_otherClass == 0:
            return 0
        else: 
            return float(self.T_myClass) / float(self.T_myClass + self.F_otherClass)
    
    
    def getClassSpecificity(self):
        """ Return class specificity i.e. Measure of the ability to correctly identify members not of this class."""
        if self.T_otherClass + self.F_myClass == 0:
            return 0
        else: 
            return float(self.T_otherClass) / float(self.T_otherClass + self.F_myClass)
        
        
    def reportClassAccuracy(self):
        """ Print to standard out, summary on the class accuracy. """
        print "-----------------------------------------------"
        print "TP = "+str(self.T_myClass)
        print "TN = "+str(self.T_otherClass)
        print "FP = "+str(self.F_myClass)
        print "FN = "+str(self.F_otherClass)
        print "-----------------------------------------------"