"""
Name: EK_AF_UCS_ConfigPars.py
Author: Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA
Contact: ryan.j.urbanowicz@darmouth.edu
Created: Sept. 4, 2012
Description: Manages the Configuration File, by loading it, parsing it and passing values to the Constants module.
---------------------------------------------------------------------------------------------------------------------------------------------------------
EK_AF_UCS Version 2.0: An expanded learning classifier system algorithm implemented with our new rule compaction strategies for data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

from EK_AF_UCS_Constants import *

class ConfigParser:
    def __init__(self, filename):
        self.commentChar = '#'
        self.paramChar =  '='
        self.parameters = self.parseConfig(filename) #Parse the configuration file and get all parameters.
        cons.setConstants(self.parameters) #Begin building constants class using parameters from configuration file
        
 
    def parseConfig(self, filename):
        parameters = {}
        f = open(filename)
        for line in f:
            # First, remove comments:
            if self.commentChar in line:
                # split on comment char, keep only the part before
                line, comment = line.split(self.commentChar, 1)
            # Second, find lines with an parameter=value:
            if self.paramChar in line:
                # split on parameter char:
                parameter, value = line.split(self.paramChar, 1)
                # strip spaces:
                parameter = parameter.strip()
                value = value.strip()
                # store in dictionary:
                parameters[parameter] = value
                
        f.close()
        return parameters