"""
Name: EK_AF_UCS_Constants.py
Author: Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA
Contact: ryan.j.urbanowicz@darmouth.edu
Created: Sept. 4, 2012
---------------------------------------------------------------------------------------------------------------------------------------------------------
EK_AF_UCS Version 2.0: An expanded learning classifier system algorithm implemented with our new rule compaction strategies for data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

from EK_AF_UCS_ExpertKnowledge import *

class Constants:
    def setConstants(self,par):
        """ Takes the parameters parsed as a dictionary in EK_AF_UCS_ConfigParser and makes these parameters available throughout EK_AF_UCS. """
        self.trainFile = par['trainFile']                               #Saved as text
        self.testFile = par['testFile']                                 #Saved as text
        self.outFileName = str(par['outFileName'])+'_EK_AF_UCS'          #Saved as text
        #----------------------------------------------------------------------------------
        self.labelInstanceID = par['labelInstanceID']                   #Saved as text
        self.labelClass = par['labelClass']                             #Saved as text
        self.discreteAttributeLimit = int(par['discreteAttributeLimit'])#Saved as int
        self.labelMissingData = par['labelMissingData']                 #Saved as text
        self.alleleSeparation = par['alleleSeparation']                 #Saved as text
        self.outputAttCoOccur = bool(int(par['outputAttCoOccur']))      #Saved as Boolean
        #----------------------------------------------------------------------------------
        self.trackingFrequency = int(par['trackingFrequency'])      #Saved as int
        self.learningIterations = par['learningIterations']         #Saved as text
        
        self.N = int(par['N'])                                      #Saved as int
        self.nu = int(par['nu'])                                    #Saved as int
        self.chi = float(par['chi'])                                #Saved as float
        self.upsilon = float(par['upsilon'])                        #Saved as float
        self.theta_GA = int(par['theta_GA'])                        #Saved as int
        self.p_spec = float(par['p_spec'])                          #Saved as float
        self.init_fit = float(par['init_fit'])                      #Saved as float
        
        self.theta_sel = float(par['theta_sel'])                    #Saved as float
        
        self.doSubsumption = bool(int(par['doSubsumption']))                #Saved as Boolean
        self.selectionMethod = par['selectionMethod']                       #Saved as text
        self.crossoverMethod = par['crossoverMethod']                       #Saved as text
        self.fitnessMethod = par['fitnessMethod']                           #Saved as text
        
        self.doAttributeTracking = bool(int(par['doAttributeTracking']))    #Saved as Boolean
        self.doAttributeFeedback = bool(int(par['doAttributeFeedback']))    #Saved as Boolean
        self.adjustedAT = bool(int(par['adjustedAT']))                #Saved as Boolean
        
        self.useExpertKnowledge = bool(int(par['useExpertKnowledge']))      #Saved as Boolean
        self.EK_source = par['EK_source']                                   #Saved as text
        self.EK_max = float(par['EK_max'])                                  #Saved as float
        self.EK_dop = int(par['EK_dop'])                                    #Saved as int

        self.doRuleCompaction = bool(int(par['doRuleCompaction']))          #Saved as Boolean
        self.onlyRC = bool(int(par['onlyRC']))                              #Saved as Boolean
        self.ruleCompactionMethod = par['ruleCompactionMethod']             #Saved as text

        self.doPopulationReboot = bool(int(par['doPopulationReboot']))      #Saved as Boolean
        self.popRebootPath = par['popRebootPath']                           #Saved as text

        self.doArtificialNoise = bool(int(par['doArtificialNoise']))        #Saved as Boolean
        self.noiseMagnitude = float(par['noiseMagnitude'])                  #Saved as float
        self.noiseWeightType = par['noiseWeightType']                           #Saved as text
    
        self.firstEpochComplete = False
        
        
    def referenceTimer(self, timer):
        """ Store reference to the timer object. """
        self.timer = timer
        
        
    def referenceEnv(self, e):
        """ Store reference to environment object. """
        self.env = e


    def referenceAttributeTracking(self, AT):
        """ Store reference to attribute tracking object. """
        self.AT = AT
        
    
    def makeExpert(self): 
        """ Make an expert Knowledge object and make an attribute weight list available via cons.EKweights"""
        EKobject = ExpertKnowledge(self.EK_source, self.EK_max, self.EK_dop, self.p_spec)
        self.EK = EKobject.getEKWeights()


    def updateRAINWeights(self, weights):
        """Updates new noise weight lists for RAIN mechanism."""
        self.RAINWeights = weights
        
        
    def completeEpoch(self):
        """ Indicate when a the first learning epoch has completed. """
        self.firstEpochComplete = True
        
        
    def parseIterations(self):
        #Construct Additional Constants---------------------------------------------------------------------------------------------------------------------
        checkpoints = self.learningIterations.split('.') #Parse the string specifying evaluation checkpoints, and the maximum number of learning iterations.
        for i in range(len(checkpoints)): #Convert checkpoint iterations from strings to ints.
            checkpoints[i] = int(checkpoints[i])
        self.learningCheckpoints = checkpoints
        self.maxLearningIterations = self.learningCheckpoints[(len(self.learningCheckpoints)-1)] 
        if self.trackingFrequency == 0:
            self.trackingFrequency = self.env.formatData.numTrainInstances  #Adjust tracking frequency to match the training data size - learning tracking occurs once every epoch
        #---------------------------------------------------------------------------------------------------------------------------------------------------
        
cons = Constants() #To access one of the above constant values from another module, import GHCS_Constants * and use "cons.Xconstant"