"""
Name: EK_AF_UCS_Prediction.py
Author: Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA
Contact: ryan.j.urbanowicz@darmouth.edu
Created: Sept. 4, 2012
Description:  Based on a given match set, this module uses a voting scheme to select the class prediction for EK_AF_UCS. 
Additional Notes: Makes a random choice between tied best classes, when more than 2 classes exist in the dataset.  
---------------------------------------------------------------------------------------------------------------------------------------------------------
EK_AF_UCS Version 2.0: An expanded learning classifier system algorithm implemented with our new rule compaction strategies for data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

from EK_AF_UCS_Constants import *
import random

class Prediction:
    def __init__(self, population, size):  #now takes in population ( have to reference the match set to do prediction)  pop.matchSet
        """ Constructs the voting array """
        self.vote = {}
        for eachClass in cons.env.formatData.classList:
            self.vote[eachClass] = 0.0

        for ref in population.matchSet:
            cl = population.popSet[ref]
            self.vote[cl.classification] += cl.fitness
            
               
    #Class Selection Methods -----------------------------------------------------------
    def bestClassWinner(self):
        """ Selects the class in the prediction array with the largest vote.  Returns 'None' if the match set was empty, and returns
        'tie' if two or more classes tied with the largest vote. """
        highVal = 0.0
        bestClass = [] #Prediction is set up to handle best class ties for problems with more than 2 classes
        for thisClass in cons.env.formatData.classList:
            if self.vote[thisClass] >= highVal:
                highVal = self.vote[thisClass]
                
        for thisClass in cons.env.formatData.classList:
            if self.vote[thisClass] == highVal: #Tie for best class
                bestClass.append(thisClass)

        #---------------------------
        if highVal == 0.0:
            return None
        elif len(bestClass) > 1: #Randomly choose between the best tied classes
            return random.choice(bestClass)
        else: #One best class
            return bestClass[0]
        #---------------------------