"""
Name: EK_AF_UCS_ExpertKnowledge.py
Author: Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA
Contact: ryan.j.urbanowicz@darmouth.edu
Created: Sept. 4, 2012
Description:  Module for developing and testing EK_AF_UCS algorithm.  This module will run EK_AF_UCS directly within an editor (e.g. Eclipse with PyDev)
---------------------------------------------------------------------------------------------------------------------------------------------------------
EK_AF_UCS Version 2.0: An expanded learning classifier system algorithm implemented with our new rule compaction strategies for data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

#Import Required Modules
from EK_AF_UCS_Timer import Timer
from EK_AF_UCS_ConfigParser import ConfigParser
from EK_AF_UCS_Environment import Environment
from EK_AF_UCS_Algorithm import EK_AF_UCS
from EK_AF_UCS_Constants import *
from EK_AF_UCS_AttributeTracking import AttributeTracking

helpstr = """
Failed attempt to run EK_AF_UCS.
"""

#Obtain path to configuration file
configurationFile = "EK_AF_UCS_Configuration_File_Example.txt"

#Initialize the Parameters object - this will parse the configuration file and store all constants and parameters.
ConfigParser(configurationFile)

#Engage Timer - tracks run time of algorithm and it's components.
timer = Timer() #TIME
cons.referenceTimer(timer)

#Initialize the Environment object - this manages the data presented to EK_AF_UCS 
env = Environment()
cons.referenceEnv(env) #Send reference to environment object to constants - to access from anywhere in EK_AF_UCS
cons.parseIterations() 

#Instantiate EK_AF_UCS Algorithm
algorithm = EK_AF_UCS()

if cons.onlyRC:
    algorithm.runRConly()
else: #Run the EK_AF_UCS algorithm.
    if cons.useExpertKnowledge: #Transform EK scores into probabilities weights for covering. Done once. EK must be externally provided.
        cons.timer.startTimeEK()
        cons.makeExpert() #stores the calculated EK probabilities in EK_AF_UCS_Constants.
        cons.timer.stopTimeEK()
        
    if cons.doAttributeTracking:
        cons.timer.startTimeAT()
        AT = AttributeTracking(True)
        cons.timer.stopTimeAT()
    else:
        AT = AttributeTracking(False)
    cons.referenceAttributeTracking(AT)

    algorithm.runEK_AF_UCS()